# uploadCopybook.pl - Upload a Copybook file
#
# Authors:  Jane Huang
# Last updated:  3/22/07
#                Xiao Lin Zhang, 8/17/09
#
# (c) Copyright IBM Corp. 2008
#
# The common routines used in all the build scripts are contained in the 
# bccpbmod.pm module.
#
# This script is spawned from the Copybook_main.pl script for each Copybook
# file that was selected to be uploaded.  
#
# The calling syntax for this script is:
# 	Usage: uploadCopybook.pl [-1] -d directory_path -f filename
# Where;
# 	-1 indicates that the uploading list is one file which causes
#          the log output to be displayed to stdout as well as the log file
# 	-d identifies the directory where the script and logs exist
# 	-f identifies the file to be uploaded
#
#
# This  script can be limited to specific users during development
# using the following lines and changing the username strings.
# my $user = $ENV{'USERNAME'};
# exit 0 if ((lc $user ne "sseifert") && (lc $user ne "brandt\.onorato"));
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# * [instructions here]                                                        *
# ****************************************************************************** 
use File::Basename;
use File::Copy;
use POSIX qw(strftime);
use Getopt::Std;
use bccpbmod;

my $debug      = $bccpbmod::debug;     # for verbose debugging output
my $debug      = 1;     # for verbose debugging output

my $rccVerbosity = "";
$rccVerbosity = '-V -V -V' if $debug;

my $sysout       = $bccpbmod::sysout;
$sysout = 'RCCEXT=RCCOUT,DISP=(NEW,CATLG,DELETE),SPACE=(32000,(30,30))' if $debug;

my $jcl;	    # JCL filename
my $ncpycnt = 7;    # number of CICS NewCopy steps
my $lcnt    = 0;    # general loop counter

# #############################################################################
# process input arguments
#
#
my $usage = "uploadCopybook [-1] -d dir -f file";


my $compile_1 = 0; # Compilation from main consisted of one file
my $dir_file  = ''; # directory where the file resides
my $filename  = ''; # filename - full or relative path

if (!getopts('1d:f:') || $opt_h) {
    print "$usage \n";
    exit(0);
}


$compile_1 = $opt_1;
$dir_file  = $opt_d;
$filename  = $opt_f;

my $current_dir = `cd`;
print "Upload script: current dir =  $current_dir \n" if $debug;

print "Upload script: One file = $compile_1 \n" if $debug;
print "Upload script: filename = $filename \n" if $debug;
print "Upload script: dir_file = $dir_file \n" if $debug;


# #############################################################################
# Determine the directory context for the execution
#
if ($dir_file ne '') {
    # directory of file specified, cd to there
    print "Changing directory from $current_dir to $dir_file\n" if $debug;
    chdir($dir_file);

    $current_dir = `cd`;
    print "current dir =  $current_dir \n" if $debug;
}

# #############################################################################
# Retrieve the environment information from the CC Project attribute
#
# *******************************************************************************
# * Optional customization:                                                     *
# * You can use the optional variables or add new variables based on your site's*
# * needs.  If you determine to use the optional variables or add new variables,*
# * you have to update all related scripts correspondently.                     *
# *******************************************************************************
my ($env, $db2sys, $menv, $os390_hostname, $os390_port, $stage,
    $copylib1, $copylib2, $copylib3, $copylib4,
    $syslib1, $syslib2, $syslib3,
#    $syslib4, $steplib1, $steplib2,			 # Optional
#    $hostpds_prefix, $hostpds_affix, $cbllib,	         # Optional 
    $loadlib, $c390llib, $dbrmlib, $tgtlib, $db2_owner) =
  bccpbmod::setEnvironmentVar();


# print "Env: $env \n" if $debug;
# print "db2sys: $db2sys \n" if $debug;

# #############################################################################
# Translate the distributed username to a mainframe ID
#
my $pcid = $ENV{USERNAME};
my $mfid = bccpbmod::getUserID($pcid);   
             
bccpbmod::my_die('Unknown User.') if ($mfid eq "");   


# #############################################################################
# Set the local copylib and jcl/log directory for the current user
#
# *******************************************************************************
# * Optional customization:                                                     *
# * You may hardcode PDS with a prefix or an affix.                             *
# *******************************************************************************
#my $name = "MFX." . $mfid;       # hardcode PDS with a prefix => MFX.USERID
my $name = $mfid . ".A";          # hardcode PDS with a affix  => USERID.A

# *******************************************************************************
# * Optional customization:                                                     *
# * The following code can be used to generate the PDS if the type attribute    *
# * ZCCENV is set with the $hostpds_prefix and $hostpds_affix.                  *
# *******************************************************************************          
#
#  my $name = $mfid;
#  my $affix ="";
#  my $prefix ="";
#  if (($hostpds_prefix ne '') || ($hostpds_affix ne '')) {
#     if ($hostpds_prefix ne '') {
#        $prefix = $hostpds_prefix . ".";
#     }
#     if ($hostpds_affix ne '') {
#        $affix = "." . $hostpds_affix ;
#     }
#     $name = $prefix . $mfid . $affix ;
#   } 
#####################################################################################

my $jcldir = $mfid;

print "Upload script: PCID: $pcid, MFID: $mfid, NAME: $name\n" if $debug;

# #############################################################################
# Cleanup JCL files from the previous execution
# bccpbmod::cleanup($jcldir);

# #############################################################################
# determine the member name from the clearcase filename. The member name is
# just the filename without the front path and extension.
#
my $delimiter= '\\';
my $del_ind  = rindex( $filename, $delimiter);
my $mem_ext  = substr( $filename, ($del_ind == -1 ? 0: ($del_ind+1)), length($filename));
my $dot      = rindex( $mem_ext, '.' );
my $mem      = uc( substr( $mem_ext, 0, $dot ) );
my $ext      = substr( $mem_ext, $dot );

printf( "Upload script: processing file (%s)\n", $filename ) if $debug;
printf( "Upload script: mem (%s) ext (%s)\n", $mem, $ext ) if $debug;

# *******************************************************************************
# * Required customization:                                                     *
# * The following code has to be modified to meet your site's requirement.      *
# *******************************************************************************
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< main >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# The main routine continues by calling each of the specific job routines.
# If any of the routines fail, this scripts exits with an error message.

my $DLL = 'F';
if ( $mem =~ /^\w{4}[S]\w{2,3}$/ ) {    # 5th Char = 'S' (Subroutine)
    $DLL = 'U';
}
# print $DLL if $debug;

bccpbmod::my_die('Allocation error.')                     if ( s1alloc() != 0 );   
bccpbmod::my_die('Upload Copybook error.')                if ( uploadcopy() != 0 );   

# Otherwise, exit successfully
exit 0;     # test only - comment out by jane


##############################################################
# The following subroutines contain each of the JCL decks
# that execute specific mainframe tasks
#
# Each subroutine performs the following functions:
#    1) build JCL file name
#    2) write specific JCL to the file
#    3) execute the JCL
#    4) report status
#
##############################################################
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< s1alloc >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
sub s1alloc {
    $jcl = 's1alloc.jcl';
    if(-f $jcl)
    {
    	unlink($jcl);
    }

    bccpbmod::writeFile(
        $jcl, $compile_1, "
//* ------------------------------------------------------------------
//S1ALLOC  EXEC PGM=IEFBR14
//*        ----
//*        ---- IEFBR14 - Upload and Allocate Temporary Files
//*        ----
//TEMP     DD  DSN=$name.XCOPYLIB,
//             DISP=(MOD,KEEP,DELETE),
//             UNIT=SYSDA,SPACE=(CYL,(5,5,5)),
//             DCB=(RECFM=FB,LRECL=80,BLKSIZE=3200)
//* ------------------------------------------------------------------
"
    );

    my $cmd =
"$bccpbmod::rccbuild -h $os390_hostname\@$os390_port -b S1ALLOC -ft $jcl  -n 0 -c LE $rccVerbosity";

    print "Step: s1alloc cmd: $cmd\n " if $debug;
    my $status = 0;

    $status = bccpbmod::rcc_system($compile_1, $cmd);
    printf( "s1alloc Status was (%d)\n", $status ) if $debug;

    move( $jcl, "./$jcldir/$jcl" );

    return $status;
}


# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< s1alloc >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
sub uploadcopy {
    $jcl = 'upload.jcl';
    if(-f $jcl)
    {
    	unlink($jcl);
    }

    bccpbmod::writeFile(
        $jcl, $compile_1, "
//* ------------------------------------------------------------------
//UPLOAD   EXEC PGM=IEFBR14
//*        ----
//*        ---- IEFBR14 - Upload and Allocate Copybook Files
//*        ----
//UPLD1    DD  DSN=$name.XCOPYLIB,RCCEXT=CPY,
//             DISP=SHR
//* ------------------------------------------------------------------
"
    );

    my $cmd =
"$bccpbmod::rccbuild -h $os390_hostname\@$os390_port -b UPLOAD -ft $jcl -i $filename -n 0 -c LE $rccVerbosity";

    print "Step: s1alloc cmd: $cmd\n " if $debug;
    my $status = 0;

    $status = bccpbmod::rcc_system($compile_1, $cmd);
    printf( "upload Status was (%d)\n", $status ) if $debug;

    move( $jcl, "./$jcldir/$jcl" );

    return $status;
}

